#include "../../libs/protocol/protocol.h"
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

int main() {
	const char *message_queue_path = "/djumbai/fifos/message_queue";

	if (faccessat(AT_FDCWD, message_queue_path, F_OK, AT_EACCESS) != -1) {
		// FIFO exists, delete it
		if (unlink(message_queue_path) == -1) {
			perror("unlink");
			exit(EXIT_FAILURE);
		}
		printf("Existing FIFO deleted.\n");
	}

	// Open the FIFO for reading
	if (mkfifo(message_queue_path, 0660) == -1) {
		perror("mkfifo");
		exit(EXIT_FAILURE);
	}
	int message_queue_fd;
	message_queue_fd = open(message_queue_path, O_RDWR);
	if (message_queue_fd == -1) {
		if (errno == ENOENT) {
			// FIFO does not exist
			printf("FIFO '%s' does not exist. Exiting...\n", message_queue_path);
			return 1;
		} else {
			perror("open:");
			return 1;
		}
	}

	const char *send_fifo_path = "/djumbai/fifos/send_fifo";

	// Check if I have write permissions on the FIFO
	if (faccessat(AT_FDCWD, send_fifo_path, W_OK, AT_EACCESS) != -1) {
		printf("You have write access to the FIFO.\n");
	} else {
		perror("Error");
		exit(EXIT_FAILURE);
	}

	int send_fifo_fd;
	// Open the FIFO for writing
	send_fifo_fd = open(send_fifo_path, O_WRONLY);
	if (send_fifo_fd == -1) {
		if (errno == ENOENT) {
			// FIFO does not exist
			printf("FIFO '%s' does not exist. Exiting...\n", send_fifo_path);
			return 1;
		} else {
			perror("open");
			return 1;
		}
	}

	unsigned char buffer[MESSAGE_SIZE];
	while (1) {
		read(message_queue_fd, buffer, MESSAGE_SIZE);
		write(send_fifo_fd, buffer, MESSAGE_SIZE);
		memset(buffer, 0, MESSAGE_SIZE);
	}

	close(message_queue_fd);
	unlink(message_queue_path);
	close(send_fifo_fd);
}
